#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <sys/socket.h>
#include <time.h>
#include <signal.h>

/*
#include "utils.h"
#include "findsaddr.h"
#include "ipfrag.h"
#include "planetlab.h"
*/

#include <err.h>
#include "utils.h"
//#include "config.h"
#include "dnet.h"
#include "aton.h"

#include "rawdnet.h"

RawDNet::RawDNet()
{
	init();
}

RawDNet::RawDNet(int ptype)
{
	init();
	ptype_ = ptype;
}

RawDNet::~RawDNet()
{
	/*
	if(frame_frags_) {
		assert(frag_num_ > 1);
		for(int i=1; i<frag_num_; i++) {
			delete frame_frags_[i];
			frame_frags_[i] = 0;
		}
		delete frame_frags_;
		delete frameslen_;
	}
	*/

	if(eth_ && eth_created_)
		eth_close(eth_);
}

void RawDNet::releaseFrags()
{
	if(frame_frags_) {
		assert(frag_num_ > 1);
		for(int i=1; i<frag_num_; i++) {
			delete frame_frags_[i];
			frame_frags_[i] = 0;
		}
		delete frame_frags_;
		frame_frags_ = NULL;
		delete frameslen_;
		frameslen_ = NULL;
	}
	frag_num_ = 1;
	frag_ = 0;
}

void RawDNet::init()
{
	// eth header part
	memset(eth_data_, 0, IP_LEN_MAX+ETH_HDR_LEN);
	sprintf(eth_src_, "00:C0:F0:75:E0:68");
	sprintf(eth_dst_, "00:D0:05:2C:98:00");
	//sprintf(eth_dst_, "00:0D:56:A4:49:17");
	beth_init = false;
	
	// IP header part
	ptype_ = RAW_PACKET_ICMP;
	pktlen_ = 100;
	ip_data_ = eth_data_ + ETH_HDR_LEN;

	char myhost[MAXHOSTNAMELEN];
	gethostname(myhost, MAXHOSTNAMELEN);	
/*	
	struct hostinfo *hi;
	hi = gethostinfo(myhost);
	setsin(&src_, hi->addrs[0]);

	hi = gethostinfo("165.124.182.78");
	setsin(&dst_, hi->addrs[0]);
*/

	ident_ = 0;
	ttl_ = 128;
	frag_ = 0;
	offset_ = 0;
	optlen_ = 0;
	
	proto_ = IPPROTO_ICMP;

	// 
	framelen_ = pktlen_ - IP_HEADER_SIZE;
	app_data_ = ip_data_ + IP_HEADER_SIZE;
		
	// ICMP header
	itype_ = ICMP_ECHO;
	icode_ = 0;
	iid_ = 0;
	iseq_ = 0;

	// UDP header
	usport_ = DEFAULT_UDP_SPORT;
	udport_ = DEFAULT_UDP_DPORT;	

	// TCP header
	tsport_ = DEFAULT_TCP_SPORT;	
	tdport_ = DEFAULT_TCP_DPORT;	
	tseq_ = 0;
	taseq_ = 0;
	toffset_ = 0;
	tsyn_ = 0;
	tack_ = 1;
	tfin_ = 0;
	twin_ = 1024*8;
	turgent_ = 0;

	//Fragmentations
	frag_num_ = 1;
	frame_frags_ = NULL;
	frameslen_ = NULL;

	eth_ = 0;	
	eth_created_ = false;
}

bool RawDNet::set_dst(char *dst_hostname)
{	
	dst_.sin_family = AF_INET;
	return gethostaddr(dst_hostname, &dst_);
}

bool RawDNet::set_src(char *src_hostname)
{	
	src_.sin_family = AF_INET;
	return gethostaddr(src_hostname, &src_);
}

void RawDNet::set_pktlen(int pktlen)
{	
	pktlen_ = pktlen;
	framelen_ = pktlen - IP_HEADER_SIZE;
}


void RawDNet::set_tflag(int flag)
{
	tsyn_ = 0;
	tack_ = 0;
	tfin_ = 0;

	if(flag & TH_ACK)
		tack_ = 1;
	if(flag & TH_FIN)
		tfin_ = 1;
	if(flag & TH_SYN)
		tsyn_ = 1;
}

void RawDNet::set_reply_id(int id)
{
	switch(ptype_) {
	case IPPROTO_ICMP:
		iid_ = id;
		break;
	case IPPROTO_UDP:
		udport_ = id;
		break;
	case IPPROTO_TCP:
		tdport_ = id;
		break;	
	}
}

void RawDNet::get_id_and_seq(int &id, int &seq)
{
	switch(ptype_) {
	case IPPROTO_ICMP:
		id = iid_;
		seq = iseq_;
		break;
	case IPPROTO_UDP:
		id = udport_;
		seq = usport_;
		break;
	case IPPROTO_TCP:
		id = tdport_;
		seq = tsport_;
		break;	
	}	
}

int &RawDNet::get_seq()
{
	switch(ptype_) {
	case IPPROTO_ICMP:
		return iseq_;
		break;
	case IPPROTO_UDP:
		return usport_;
		break;
	case IPPROTO_TCP:
		return tsport_;
		break;	
	}
	return seq_;	// no use
}

int &RawDNet::get_id()
{
	switch(ptype_) {
	case IPPROTO_ICMP:
		return iid_;
		break;
	case IPPROTO_UDP:
		return udport_;
		break;
	case IPPROTO_TCP:
		return tdport_;
		break;	
	}
	return id_;	// no use
}

void RawDNet::save_id_seq()
{	
	saved_ipid_ = ident_;
	switch(ptype_) {
	case IPPROTO_ICMP:
		seq_ = iseq_;
		id_ = iid_;
		break;
	case IPPROTO_UDP:
		seq_ = usport_;
		id_ = udport_;
		break;
	case IPPROTO_TCP:
		seq_ = tsport_;
		id_ = tdport_;
		break;	
	}
}

void RawDNet::load_id_seq()
{	
	ident_ = saved_ipid_;
	switch(ptype_) {
	case IPPROTO_ICMP:
		iseq_ = seq_;
		iid_ = id_;
		break;
	case IPPROTO_UDP:
		usport_ = seq_;
		udport_ = id_;
		break;
	case IPPROTO_TCP:
		tsport_ = seq_;
		tdport_ = id_;
		break;	
	}
}

int RawDNet::next_pkt()
{
	return next_pkt(1);
}

int RawDNet::next_pkt(int n)
{
	ident_+= n;
	switch(ptype_) {
	case IPPROTO_ICMP:
		iseq_ += n;
		return iid_;
		break;
	case IPPROTO_UDP:
		usport_ += n;
		return udport_;
		break;
	case IPPROTO_TCP:
		tsport_ += n;
		return tdport_;
		break;	
	}
}


bool RawDNet::open_eth(char *peth)
{
	// devices
	if ((eth_ = eth_open(peth)) == NULL) {
		err(1, "eth_open");
		return false;
	}
	eth_created_ = true;
	return true;
}

int RawDNet::send_dnet()
{
	eth_t *eth;
	ip_t *ip;
	int len;
	
	if (eth_ == NULL) {
		len = pktlen_;
	/*	for(int i=0; i<len; i++) {
			u_char c = ip_data_[i];
			printf("%x ", c);
			if(i%16==15)
				printf("\n");
		}
		printf("\n");
	*/	
		if ((ip = ip_open()) == NULL)
			err(1, "ip_open");
		if (ip_send(ip, ip_data_, len) != len)
			err(1, "ip_send");
		ip_close(ip);
		//printf("send ip\n");
	} else {
		if(beth_init == false)
			eth_init();
		len = pktlen_ + ETH_HDR_LEN;
//		if ((eth = eth_open(peth)) == NULL)
//			err(1, "eth_open");
		if (eth_send(eth_, eth_data_, len) != len)
			err(1, "eth_send");
//		eth_close(eth);
		//printf("send eth\n");
	}
	return 0;	
}

int RawDNet::eth_init()
{
	struct eth_hdr *eth;
	struct addr addr;
	u_char *p;
	char *name, *value;
	int c, len;

	eth = (struct eth_hdr *)eth_data_;
	memset(eth, 0, sizeof(*eth));
	eth->eth_type = htons(ETH_TYPE_IP);

	// source MAC address
	if (addr_aton(eth_src_, &addr) < 0) 
		fprintf(stderr, "eth_init: wrong src MAC: %s", eth_src_);
	memcpy(&eth->eth_src, &addr.addr_eth, ETH_ADDR_LEN);
	// gateway MAC address
	if (addr_aton(eth_dst_, &addr) < 0)
		fprintf(stderr, "eth_init: wrong dst MAC: %s", eth_dst_);
	memcpy(&eth->eth_dst, &addr.addr_eth, ETH_ADDR_LEN);		

	beth_init = true;
	return (0);
}

void RawDNet::init_ip()
{
	IP_HDR *outip;
	int len;

	outip = (IP_HDR *)ip_data_;
	//bzero((char *)ip_data_, pktlen_);
	
	outip->ip_v = 4;
	outip->ip_hl = sizeof(IP_HDR) >> 2;
	outip->ip_tos = 0;
	outip->ip_len = htons(pktlen_);
	outip->ip_ttl = ttl_;
	//outip->ip_off = htons(IP_DF);
	if(frag_ == 0) {
		outip->ip_off = htons(IP_DF);
	}
	else if(frag_ > 0) {     // not the last fragment
		//outip->ip_off = htons(IP_MF);
		outip->ip_off = IP_MF;
		outip->ip_off |= (IP_OFFMASK & (offset_/8));
		outip->ip_off = htons(outip->ip_off);
	}
	else if(frag_ < 0) {     // The last fragment
		outip->ip_off = 0;
		outip->ip_off |= (IP_OFFMASK & (offset_/8));
		outip->ip_off = htons(outip->ip_off);
	}		
	outip->ip_id = htons(ident_);
	outip->ip_p = ptype_;		
	//outip->ip_src = src_.sin_addr;
	//outip->ip_dst = dst_.sin_addr;
	outip->ip_src = src_.sin_addr.s_addr;
	outip->ip_dst = dst_.sin_addr.s_addr;
	
	len = pktlen_;
	outip->ip_len = htons(len);
	
	ip_checksum(ip_data_, len);
}

bool RawDNet::init_icmp()
{
	ICMP_HDR *outicmp;
	ICMP_ECHO_HDR *outicmpecho;
	ICMP_TSTAMP_HDR *outicmptstamp;
	char *outdata;
	int i, j;
	
	outicmp = (ICMP_HDR *)app_data_;
	outicmpecho = (ICMP_ECHO_HDR *)(outicmp+1);
	outicmptstamp = (ICMP_TSTAMP_HDR *)(outicmp+1); 

	outicmp->icmp_type = itype_;
	outicmp->icmp_code = icode_;

	if(itype_ == ICMP_ECHO) {		// ICMP ECHO
		//printf("iid_:%d\n", iid_);
		outicmpecho->icmp_id = htons(iid_);
		outicmpecho->icmp_seq = htons(iseq_);
		
		//outdata = (char *)(outicmpecho+1);
		outdata = (char *)outicmpecho->icmp_data;
		for(i = 0, j =0; i<framelen_ - sizeof(ICMP_HDR); i++, j++, j%=26) {
			outdata[i] = 'a' + j;
		}

		// To bypass the limited 16bit ICMP seq number
		int *payload_seq = (int *)outdata;
		*payload_seq = iseq_;
	}
	else if(itype_ == ICMP_TSTAMP) {
		outicmptstamp->icmp_id = htons(iid_);
		outicmptstamp->icmp_seq = htons(iseq_);		
		outicmptstamp->icmp_ts_orig = htonl(100);
		outicmptstamp->icmp_ts_rx = htonl(0);
		outicmptstamp->icmp_ts_tx = htonl(0);
		
		outdata = (char *)(outicmptstamp+1);		
		for(i = 0, j =0; i<framelen_ - sizeof(ICMP_HDR); i++, j++, j%=26) {
			outdata[i] = 'a' + j;
		}
		//printf("iseq_: %d\n", ntohs(outicmptstamp->icmp_seq));
	}
	else {
		return false;
	}
	outicmp->icmp_cksum = 0;
	
/*
	// ICMP checksum
	outicmp->icmp_cksum = in_cksum((u_short *)outicmp, pktlen_ - (sizeof(IP_HDR) + optlen_));
	if (outicmp->icmp_cksum == 0)
		outicmp->icmp_cksum = 0xffff;	
*/

	return true;
}

bool RawDNet::init_udp()
{
	UDP_HDR *outudp;
	IP_HDR *outip, tip;
	struct udpiphdr *ui, *oui;
	char *outdata;
	int i, j;
	
	outudp = (UDP_HDR *)app_data_;
	
	outudp->uh_sport = htons(usport_);
	outudp->uh_dport = htons(udport_);
	outudp->uh_ulen = htons((u_short) framelen_);
	outudp->uh_sum = 0;
	
	outdata = (char *)(outudp+1);
	for(i = 0, j =0; i<framelen_ - sizeof(UDP_HDR); i++, j++, j%=26) {
		outdata[i] = 'a' + j;
	}

	return true;
}

bool RawDNet::init_tcp()
{
	TCP_HDR *outtcp;
	struct psd_hdr phdr;
	IP_HDR *outip, tip;
	struct tcpiphdr *ti, *oti;
	char buff[65536];

	outip = (IP_HDR*)ip_data_;
	outtcp = (TCP_HDR *)app_data_;
	memset(outtcp, 0, sizeof(TCP_HDR));
/*
	outtcp->source = htons(tsport_);
	outtcp->dest = htons(tdport_);	
	outtcp->seq = htonl(tseq_);
	outtcp->ack_seq = htonl(taseq_);
	outtcp->doff = 5;		// 5*4 = 20 bytes
	outtcp->res1 = 0;
	outtcp->res2 = 0;
	outtcp->syn = tsyn_;
	outtcp->ack = tack_;
	outtcp->fin = tfin_;
	outtcp->rst = 0;
	outtcp->psh = 0;
	outtcp->urg = 0;
	outtcp->window = htons(twin_);
	outtcp->urg_ptr = 0;
	outtcp->check = 0;
*/

	outtcp->th_sport = htons(tsport_);	
	outtcp->th_dport = htons(tdport_);
	outtcp->th_seq = htonl(tseq_);
	outtcp->th_ack = htonl(taseq_);
	outtcp->th_off = 5;		// 5*4 = 20 bytes
	outtcp->th_flags = tsyn_<<1 | tack_ <<4 | tfin_ ;
	outtcp->th_win = htons(twin_);
	outtcp->th_urp = 0;
	outtcp->th_sum = 0;
//printf("tsport: %d\n", tsport_);

	return true;
}

bool RawDNet::prepare_packet()
{
	bool bret = true;

	//bzero((char *)eth_data_, sizeof(eth_data_));
	bzero((char *)ip_data_, IP_LEN_MAX);
	
	if(ptype_ == RAW_PACKET_ICMP)
		bret = init_icmp();
	else if(ptype_ == RAW_PACKET_UDP)
		bret = init_udp();
	else if(ptype_ == RAW_PACKET_TCP) {		
		bret = init_tcp();		
	}

	init_ip();
	return bret;
}

bool RawDNet::prepare_packet_frag(int frag_pkt_len)
{
	bool bret;
	releaseFrags();
	if( bret = prepare_packet() ) {
		split_packet(frag_pkt_len);
		return true;
	}
	else
		return false;
}

bool RawDNet::prepare_packet_frag(int pktlen, int frag_pkt_len)
{
	bool bret;
	set_pktlen(pktlen);
	releaseFrags();
//	tsport_ += MAX_PROBES;		// Make the source port of whole and fragmented packets different
	bret = prepare_packet();
//	tsport_ -= MAX_PROBES;
	if(bret) {
		split_packet(frag_pkt_len);
		return true;
	}
	else
		return false;
}

void RawDNet::split_packet(int frag_pkt_len)
{
	// assume now framelen_ = pktlen_ + IP_HEADER_SIZE
	int frag_frame_len = frag_pkt_len - IP_HEADER_SIZE;
	int i, len, vlen = 0;
		
	frag_frame_len = ((frag_frame_len+7)/8)*8;						// frag frame length must be a multiple of 8
	frag_num_ = (framelen_ + (frag_frame_len-1) ) / frag_frame_len;		// number of fragmentations	
	frame_frags_ = new u_char *[frag_num_];
	frameslen_ = new int[frag_num_];

	//printf("frag_pkt_len:%d, frag_frame_len:%d, frag_num_:%d, framelen_:%d\n", frag_pkt_len, frag_frame_len, frag_num_,framelen_);
	for(i=0; i<frag_num_; i++) {
		if(i == frag_num_-1)
			len = framelen_ - frag_frame_len * i;
		else
			len = frag_frame_len;
		vlen += len;
		frame_frags_[i] = new u_char[len + IP_HEADER_SIZE + ETH_ADDR_LEN];		
		frameslen_[i] = len;
		memcpy(frame_frags_[i], app_data_+ frag_frame_len * i, len);		
	//	printf("len: %d, frameslen_[i]:%d\n", len, frameslen_[i]);
	}	
//	for(i=0; i<frag_num_; i++)
//		printf("%d\n", frameslen_[i]);

	assert(vlen == framelen_);
}


int RawDNet::send_frag(int fragid)
{
	int i;
	//printf("in send_frag: fragid: %d\n", fragid);
	assert(frag_num_ >= 1 && fragid >= 1 && fragid <= frag_num_);
	if( frag_num_ == 1 ) {
		assert(fragid == 1);
		offset_ = 0;
		frag_ = 0;
		return send_dnet();
	}

	offset_ = 0;
	for(i = 0; i<fragid-1; i++)
		offset_ += frameslen_[i];		
	if(fragid < frag_num_) {		// 
		frag_ = fragid;		
	}
	else if(fragid == frag_num_)
		frag_ = -fragid;

	set_framelen(frameslen_[fragid-1]);

	//printf("%p, %p, %d\n", app_data_, frame_frags_[fragid-1], frameslen_[fragid-1]);
	memcpy(app_data_, frame_frags_[fragid-1], frameslen_[fragid-1]);
	
	init_ip();
	send_dnet();
	return 0;
}
