#ifndef __RAW_PACKET__
#define __RAW_PACKET__

/*
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <netinet/ip_icmp.h>
#include <netinet/udp.h>
#include <netinet/udp_var.h>
#include <netinet/tcp.h>
#include <netinet/tcpip.h>
*/

#include "dnet.h"

#define RAW_PACKET_ICMP	1
#define RAW_PACKET_UDP	17
#define RAW_PACKET_TCP		6

#define MAX_PROBES	5000

#define DEFAULT_TCP_SPORT	10000
#define DEFAULT_UDP_SPORT	20000

#define DEFAULT_ICMP_PORT		33449
#define DEFAULT_UDP_DPORT		(DEFAULT_ICMP_PORT+2)
#define DEFAULT_TCP_DPORT		(DEFAULT_UDP_DPORT+1)

//#define DEFAULT_ICMP_PORT  10000
//#define DEFAULT_TCP_SPORT	43470
//#define DEFAULT_UDP_SPORT	33470
//#define DEFAULT_TCP_DPORT	20000
//#define DEFAULT_UDP_DPORT	30000

typedef struct ip_hdr 		IP_HDR;
typedef struct icmp_hdr	ICMP_HDR;
typedef struct icmp_msg_echo	ICMP_ECHO_HDR;
typedef struct icmp_msg_quote	ICMP_QUOTE;
typedef struct icmp_msg_tstamp ICMP_TSTAMP_HDR;
typedef struct udp_hdr	UDP_HDR;
typedef struct tcp_hdr		TCP_HDR;
#define IP_HEADER_SIZE		sizeof(IP_HDR)

typedef struct psd_hdr 
{ 
	unsigned long saddr; 
	unsigned long daddr; 
	char mbz; 
	char ptcl; 
	unsigned short tcpl; 
}PSD_HEADER; 

class RawDNet {

public:
	RawDNet();
	RawDNet(int ptype);
	~RawDNet();

	void releaseFrags();
	
	void init_ip();
	bool init_udp();
	bool init_icmp();
	bool init_tcp();

	// Eth
	void set_eth_dst(char *dst) {	 strcpy(eth_dst_, dst); }
	void set_eth_src(char *src) 	{  strcpy(eth_src_, src); }

	// IP
	u_char *get_ip_data() { return ip_data_; }
	u_char *get_app_data() { return app_data_; }
	int	get_ptype() { return ptype_; }
	int	get_pktlen() { return pktlen_; }
	void set_pktlen(int pktlen);
	void set_framelen(int flen) { set_pktlen(flen + IP_HEADER_SIZE); };
	int &get_ident() { return ident_; }
	int &get_ttl() { return ttl_; }
	int get_fragnum() { return frag_num_; }

	bool set_src(char *src_hostname);
	void set_src(struct sockaddr_in *src) { src_ = *src; 	src_.sin_family = AF_INET; }
	bool set_dst(char *dst_hostname);
	void set_dst(struct sockaddr_in *dst) { dst_ = *dst; 	dst_.sin_family = AF_INET; }
	struct sockaddr_in *get_daddr() { return &dst_; }

	// ICMP
	int &get_icmp_itype() {	return itype_;	} 
	int &get_icmp_id() 	{	return iid_;	} 

	// TCP
	void set_tflag(int flag);

	// ALL packets
	void set_reply_id(int id);		
	void get_id_and_seq(int &id, int &seq);
	int &	get_seq();
	int &	get_id();
	int next_pkt();
	int next_pkt(int n);

// dnet functions
public:	
	bool open_eth(char *peth);
	void set_eth(eth_t *eth) { eth_ = eth; }
	int send_dnet();
	int eth_init();
	void split_packet(int frag_pkt_len);	// split a packet into fragmentations
	int send_frag(int i);	// send the ith fragmentation
	bool prepare_packet();
	bool prepare_packet_frag(int frag_pkt_len);
	bool prepare_packet_frag(int pkt_len, int frag_pkt_len);

	void save_id_seq();
	void load_id_seq();
protected:
	void init();
	
protected:
	int		id_;
	int		seq_;					// sequence number of the packet
	int		saved_ipid_;					// sequence number of the packet

	eth_t *eth_;
	bool eth_created_;
	
	// eth header part
	bool		beth_init;
	u_char	eth_data_[IP_LEN_MAX+ETH_HDR_LEN];		// eth data
	char		eth_src_[64];
	char		eth_dst_[64];

	// IP header part
	int		ptype_;					// packet type: ICMP, UDP, TCP
	int		pktlen_;					// length of the packet
	u_char*	ip_data_;
	//u_char	ip_data_[IP_LEN_MAX];	// IP data
	struct sockaddr_in src_;			// source
	struct sockaddr_in dst_;			// destination
	int		ident_;					// IP ID
	int		ttl_;						// TTL
	int		frag_;					// The flag of fragmentation. 0->DF, k->MF, -k->LAST Frag
	int		offset_;					// offset of the fragmentation
	int		proto_;					// protocol: ICMP, UDP or TCP

	int		optlen_;					// IP option length. MUST be 0 now
	
	//	UDP, TCP, ICMP part
	int		framelen_;				// The length of the frame. Maybe different if fragmentation is used
	u_char*	app_data_;
	
	// UDP header part
	int		usport_;
	int		udport_;

	// ICMP header part
	int 		itype_;					// ICMP type
	int		icode_;					// ICMP code
	int		iid_;					// ICMP ID
	int		iseq_;					// ICMP sequence number
	
	// TCP header part
	int		tsport_;	
	int		tdport_;
	int		tseq_;
	int		taseq_;
	int		toffset_;
	int		tsyn_;
	int		tack_;	
	int		tfin_;	
	int		twin_;
	int		turgent_;

	// Fragmentation
	int		frag_num_;			// Number of fragmentations
	u_char **frame_frags_;		// Fragmentations of a packet
	int*		frameslen_;			// length of frames of fragmentations
};


#endif
