#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <sys/socket.h>
#include <time.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "libnet.h"
#include <pcap.h>
#include <pcap-bpf.h>


extern char *g_device;
extern char g_rcv_packet[];
extern char *g_source;

pcap_t * g_pcap = NULL; 

bool init_pcap()
{
	char *device, errbuf[PCAP_ERRBUF_SIZE+64], *ptr; 
	struct bpf_program fcode;
	//char *filter_string = "icmp or udp";
	char filter_string[256];
	bpf_u_int32 netmask = 0;
	struct pcap_pkthdr hdr;
	struct libnet_ethernet_hdr * eth;

	struct libnet_ipv4_hdr * ipH;
	struct libnet_tcp_hdr * tcpH;

	sprintf(filter_string, "((tcp and tcp[2]>0) or icmp) and (not ip src %s) ", g_source);;
/*
	device = pcap_lookupdev(errbuf);
	if(device == 0) {
		fprintf(stderr, "pcap_lookupdev error: %s", errbuf);
		return false;
	}
	//printf("device: %s\n", device);
	if( strcmp(device, "eth0") != 0 && strcmp(device, g_device) ) {
		fprintf(stderr, "pcap_lookupdev: Wrong device %s\n", device);
		return false;
	}
*/
	device = g_device;
	if( (g_pcap = pcap_open_live(device, 65536, 1, 0, errbuf)) == NULL ) {
		fprintf(stderr, "pcap_open_live: Cannot open device %s for pcap: %s\n", device, errbuf);
		return false;
	}
	
	if( pcap_compile(g_pcap, &fcode, filter_string, 0, netmask) == -1 ) {
		fprintf(stderr, "pcap_compile error: %s\n", pcap_geterr(g_pcap) );
		return false;
	}
	if( pcap_setfilter(g_pcap, &fcode) == -1 ) {
		fprintf(stderr, "pcap_setfilter error: %s\n", pcap_geterr(g_pcap) );
		return false;
	}

	if( pcap_setnonblock(g_pcap, 1, errbuf) == -1 ) {
		fprintf(stderr, "pcap_setnonblock error: %s\n", errbuf );
		return false;
	}
	return true;
}

int getnext_pcap()
{
	struct pcap_pkthdr hdr, *phdr=&hdr;
	struct libnet_ethernet_hdr * eth;
	struct libnet_ipv4_hdr * iph;
	int len;
	char *ptr;
	//usleep(1);
	if(!g_pcap) {
		fprintf(stderr, "Pcap not initialized\n");
		return 0;
	}
	
	ptr = (char *)(pcap_next(g_pcap, &hdr));
	if(ptr == NULL) {
		return 0;
	}
	eth = (struct libnet_ethernet_hdr *)ptr;
	if(htons(eth->ether_type) == ETHERTYPE_IP) {
		
		//iph = (struct libnet_ipv4_hdr *)(ptr + 14);
		//printf("eth header length:%d\n", sizeof(struct libnet_ethernet_hdr));
		iph = (struct libnet_ipv4_hdr *)(ptr + sizeof(struct libnet_ethernet_hdr));
		len = iph->ip_len;
		memcpy(g_rcv_packet, (char *)iph, len);		
		return len;
	//	return 0;
	}

	return 0;		
}

void close_pcap()
{
	if(g_pcap)
		pcap_close(g_pcap);
}

